/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <device.h>
#include <devicetree.h>
#include <drivers/gpio.h>
#include <drivers/spi.h>

/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS   1000

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led0)

#if DT_NODE_HAS_STATUS(LED0_NODE, okay)
#define LED0	DT_GPIO_LABEL(LED0_NODE, gpios)
#define PIN	DT_GPIO_PIN(LED0_NODE, gpios)
#define FLAGS	DT_GPIO_FLAGS(LED0_NODE, gpios)
#else
/* A build error here means your board isn't set up to blink an LED. */
#error "Unsupported board: led0 devicetree alias is not defined"
#define LED0	""
#define PIN	0
#define FLAGS	0
#endif

#define PIN1 DT_GPIO_PIN(DT_ALIAS(led1), gpios)

static const struct spi_config spi_cfg = {
	.operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
		     SPI_MODE_CPOL | SPI_MODE_CPHA,
	.frequency = 4000000,
	.slave = 0,
};

const struct device * spi_dev;

static void spi_init(void)
{
	const char* const spiName = "SPI_2";
	spi_dev = device_get_binding(spiName);

	if (spi_dev == NULL) {
		printk("Could not get %s device\n", spiName);
		return;
	}
}

const struct device *dev;

static struct k_poll_signal async_sig = K_POLL_SIGNAL_INITIALIZER(async_sig);
static struct k_poll_event  async_evt =
	K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL,
				 K_POLL_MODE_NOTIFY_ONLY,
				 &async_sig);

void spi_test_send(void)
{
	int err;
	static uint8_t tx_buffer1[4] = {1,2,3,4};
	static uint8_t tx_buffer2[3] = {2,3,4};
	static uint8_t rx_buffer[2];
	static uint8_t rx_buffer2[4];

	const struct spi_buf tx_buf[2] = {{
		.buf = tx_buffer1,
		.len = sizeof(tx_buffer1)}, {
		.buf = tx_buffer2, 
		.len = sizeof(tx_buffer2)}
	};
	const struct spi_buf_set tx = {
		.buffers = tx_buf,
		.count = 1
	};

	struct spi_buf rx_buf[2] = {{
		.buf = rx_buffer,
		.len = sizeof(rx_buffer)},{
		.buf = rx_buffer2,
		.len = sizeof(rx_buffer2)}
	};
	const struct spi_buf_set rx = {
		.buffers = rx_buf,
		.count = 2
	};
	
	//err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
	err = spi_transceive_async(spi_dev, &spi_cfg, &tx, &rx, &async_sig);

	if (err) {
		printk("SPI error: %d\n", err);
		return;
	}
	// Waiting for event to occur
	err = k_poll(&async_evt, 1, K_MSEC(100));
	if (err) {
		printk("Failed waiting for SPI async event\n");
	} else {
		printk("SPI complete!\n");
		printk("TX sent: %x-%x-%x-%x\n", tx_buffer1[0], tx_buffer1[1], tx_buffer1[2], tx_buffer1[3]);
		printk("RX recv: %x-%x\n", rx_buffer[0], rx_buffer[1]);
		printk("RX2 recv: %x-%x\n", rx_buffer2[0], rx_buffer2[1]);
		tx_buffer1[0]++;
		tx_buffer1[1]++;
		tx_buffer1[2]++;
	}
}

void main(void)
{
	bool led_is_on = true;
	int ret;

	printk("SPI async test\n");

	dev = device_get_binding(LED0);
	if (dev == NULL) {
		return;
	}

	ret = gpio_pin_configure(dev, PIN, GPIO_OUTPUT_ACTIVE | FLAGS);
	if (ret < 0) {
		return;
	}

	ret = gpio_pin_configure(dev, PIN1, GPIO_OUTPUT_ACTIVE | FLAGS);
	if (ret < 0) {
		return;
	}

	spi_init();

	while (1) {
		gpio_pin_set(dev, PIN, (int)led_is_on);
		led_is_on = !led_is_on;
		spi_test_send();
		k_msleep(SLEEP_TIME_MS);
	}
}
